#include <iostream>
#include <cmath>
#include <cstring>
#include <cassert>

#define TEST(E)                                                         \
  if (!(E)) {                                                           \
    fprintf(stderr, "%s:%d: TEST NIE OK: %s\n", __FILE__, __LINE__,     \
            #E);                                                        \
    abort();                                                            \
  } else {                                                              \
    fprintf(stderr, "TEST OK: %s\n", #E);                               \
  }

// 9.1.1
int
SUMA(int a, int b)
{
  return a + b;
}

// 9.1.2
float
RÓŻNICA(int a, int b)
{
  return (float)a - (float)b;
}

// 9.1.3
float
ILOCZYN(int a, int b)
{
  return (float)a * (float)b;
}

// 9.1.4
float
ILORAZ(int a, int b)
{
  return (float)a / (float)b;
}

// 9.1.5a
bool
czy_pitagorejski(int a, int b, int c)
{
  return a*a+b*b == c*c;
}

// 9.1.5b
void
pitagorejskie()
{
  int i, j, k;
  for (i = 1; i <= 10; ++i)
    for (j = 1; j <= 10; ++j)
      for (k = 1; k <= 10; ++k)
        if (czy_pitagorejski(i, j, k))
          printf("PITAGOREJSKI: [%d %d %d]\n", i, j, k);
}

// 9.1.6
int
pierwiastek_calkowity(int x)
{
  if (x < 0)
    return -1;
  if (x == 0)
    return 0;

  int v = 1;
  while (x >= v) x -= v, v += 2;

  if (x == 0)
    return floorf((v-1.f)/2.f);
  return -1;
}

// 9.1.7
int
ile_cyfr(int x)
{
  return snprintf(NULL, 0, "%d", x);
}

// 9.1.8a
bool
czy_konczy_sie_pierwiastkiem(int x)
{
  char buf1[512], buf2[512];
  int n1 = snprintf(buf1, 512, "%.2f", (float)x);
  int n2 = snprintf(buf2, 512, "%.2f", sqrtf(x));
  int delta = n1-n2;

  assert(delta >= 0);

  return strcmp(buf1+delta, buf2) == 0;
}

// 9.1.8b
void
konczace_sie_pierwiastkiem(void)
{
  int i;
  for (i = 1; i <= 10000; ++i)
    if (czy_konczy_sie_pierwiastkiem(i))
      printf("kończąca się kwadratem: %d\n", i);
}


int
main(void)
{
  TEST(SUMA(2, 2) == 4);
  TEST(RÓŻNICA(2, 2) == 0);
  TEST(ILOCZYN(2, 3) == 6);
  TEST(ILORAZ(4, 2) == 2);
  TEST(czy_pitagorejski(3, 4, 5));
  pitagorejskie();
  TEST(pierwiastek_calkowity(4) == 2);
  TEST(ile_cyfr(2137420) == 7);
  TEST(czy_konczy_sie_pierwiastkiem(625));
  konczace_sie_pierwiastkiem();

  return 0;
}
